"""Module for copy interactors."""

import subprocess as sup
from pathlib import Path
from typing import Optional


def rsync_copy(source: Path, destination: Path, file_filter: Optional[str]):
    """Use rsync to mirror files and folders from src to dest."""
    try:
        sup.run(f"rsync -av --delete {source}/{file_filter} {destination}", shell=True)
    except FileNotFoundError:
        print(f"Unable to find source: [pink]{source}[/pink]")



"""Set functionality module."""

import subprocess as sup
from pathlib import Path
from pydantic import validate_call


@validate_call
def git_repository(
    repo: Path, remote: str, commit_msg: str, branch: str = "main", rebase: bool = False
) -> None:
    """Push latest changes in repository to remote.

    All the current edits are added and committed with the specified commit message.

    Args:
      repo: Path to the repository
      remote: Remote repository
      commit_msg: The commit message for the commit.
      branch: Branch to commit. Defaults to "master".
      rebase: If rebasing should be done when pulling. Default is False.
    """
    sup.run(["git", "-C", repo, "add", "-A"])
    sup.run(["git", "-C", repo, "commit", "-m", commit_msg])
    if rebase:
        sup.run(["git", "-C", repo, "pull", "--rebase"])
    sup.run(["git", "-C", repo, "push", remote, branch])



"""Configuration module for BACore."""

import os
from bacore.domain import settings

docker_build = os.getenv("DOCKER_BUILD", None)

project_bacore = settings.Project(
    name="BACore",
    version="0.1.4",
    description="BACore is a framework for business analysis and test automation.",
)



"""CLI Typer interface."""

from bacore.domain import files, settings
from bacore.interactors import retrieve
from pathlib import Path
from typing import Optional


class ProjectInfo:
    """ProjectInfo information."""

    def __init__(self, pyproject_file: Path):
        """Initialize."""
        self._pyproject_file_toml_object = files.TOML(path=pyproject_file)
        self._project_info_dict = retrieve.file_as_dict(
            file=self._pyproject_file_toml_object
        )
        self._project_info = settings.Project(
            name=self._project_info_dict["project"]["name"],
            version=self._project_info_dict["project"]["version"],
            description=self._project_info_dict["project"]["description"],
        )

    @property
    def name(self) -> str:
        """ProjectInfo name."""
        return self._project_info.name

    @property
    def version(self) -> Optional[str]:
        """ProjectInfo version."""
        return self._project_info.version

    @property
    def description(self) -> Optional[str]:
        """ProjectInfo description."""
        return self._project_info.description



"""Delete interactors."""

from datetime import datetime, timedelta
from pathlib import Path
from pydantic import Field, validate_call
from pydantic.dataclasses import dataclass
from typing_extensions import Annotated


@dataclass
class DeletedFiles:
    """Deleted files summary."""

    path: Path
    pattern: str
    older_than_days: int
    recursive: bool
    number_of_deleted_files: int
    deleted_files: list[str]


@validate_call
def files(
    path: Path,
    pattern: str = "*",
    older_than_days: Annotated[int, Field(ge=0)] = 0,
    recursive: bool = False,
) -> DeletedFiles:
    """Delete files older than x days.

    Args:
        path (`Path`): Path to search for files.
        pattern (`str`): Pattern to search for files.
        older_than_days (`int`): Delete files older than x dyas. Default is `0`. Negative values are not allowed.
        recursive (`bool`): Optionally delete files recursively. Default is `False`.
    """
    number_of_deleted_files = 0
    deleted_files = []

    now = datetime.now()

    find_function = path.rglob if recursive else path.glob
    for file in find_function(pattern):
        if (
            file.is_file()
            and file.stat().st_mtime
            < (now - timedelta(days=older_than_days)).timestamp()
        ):
            file.unlink()
            deleted_files.append(str(file))
            number_of_deleted_files += 1

    return DeletedFiles(
        path=path,
        pattern=pattern,
        older_than_days=older_than_days,
        recursive=recursive,
        number_of_deleted_files=number_of_deleted_files,
        deleted_files=deleted_files,
    )
